package org.linotte.frame.cahier.timbre.outils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.linotte.frame.cahier.timbre.entite.Acteur;
import org.linotte.frame.cahier.timbre.entite.BlocDebut;
import org.linotte.frame.cahier.timbre.entite.BlocFin;
import org.linotte.frame.cahier.timbre.entite.Condition;
import org.linotte.frame.cahier.timbre.entite.Pour;
import org.linotte.frame.cahier.timbre.entite.TantQue;
import org.linotte.frame.cahier.timbre.entite.Timbre;
import org.linotte.frame.cahier.timbre.entite.Verbe;
import org.linotte.frame.cahier.timbre.entite.i.PasSauvegarder;
import org.linotte.frame.cahier.timbre.ui.Planche;
import org.linotte.frame.cahier.timbre.ui.i.PlancheUI;

public class Historique {

	private int MAX = 20;

	private List<List<Timbre>> historiques = new ArrayList<>();

	private Planche planche;

	private List<Timbre> photoPlanche;

	private boolean debug = false;

	public Historique(PlancheUI plancheUI) {
		planche = (Planche) plancheUI;
	}

	public void prendrePhoto() {
		if (debug)
			System.out.println("preparationfixe");
		photoPlanche = clone(planche.timbres);
	}

	public void annulerPhoto() {
		photoPlanche = null;
	}

	public void archiverPhoto() {
		if (photoPlanche != null) {
			if (debug)
				System.out.println("fixe");
			historiques.add(photoPlanche);
			if (historiques.size() > MAX) {
				historiques.remove(0);
			}
			photoPlanche = null;
		} else {
			if (debug)
				System.out.println("fixe vide");
		}
	}

	public List<Timbre> depilerPhoto() {
		photoPlanche = null;
		if (!historiques.isEmpty())
			return historiques.remove(historiques.size() - 1);
		else
			return null;
	}

	public boolean estceVide() {
		return historiques.isEmpty();
	}

	public int taille() {
		return historiques.size();
	}

	private List<Timbre> clone(List<Timbre> timbres) {
		Map<Timbre, Timbre> livretFamille = new HashMap<>();
		// On clone :
		List<Timbre> clones = new ArrayList<Timbre>();
		for (Timbre timbre : timbres) {
			if (!(timbre instanceof PasSauvegarder)) {
				Timbre clone = timbre.clonerPourDuplication();
				livretFamille.put(timbre, clone);
				clones.add(clone);
			}
		}
		// Reconstruction des nouveaux liens avec les clones :
		for (Timbre clone : clones) {
			if (clone instanceof BlocDebut) {
				BlocDebut timbre = (BlocDebut) clone;
				if (timbre.blocfin != null)
					timbre.blocfin = (BlocFin) livretFamille.get(timbre.blocfin);
				else {
					System.out.println("blocdebut erreur " + timbre);
				}
			}
			if (clone instanceof BlocFin) {
				BlocFin timbre = (BlocFin) clone;
				if (timbre.blocDebut != null)
					timbre.blocDebut = (BlocDebut) livretFamille.get(timbre.blocDebut);
				else
					System.out.println("blocfin erreur");
			}
			if (clone instanceof Condition) {
				Condition timbre = (Condition) clone;
				if (timbre.timbreSuivantSecondaire != null)
					timbre.timbreSuivantSecondaire = (Timbre) livretFamille.get(timbre.timbreSuivantSecondaire);
				if (timbre.formule != null)
					if (livretFamille.containsKey(timbre.formule))
						timbre.formule = (Acteur) livretFamille.get(timbre.formule);
					else
						timbre.formule = (Acteur) timbre.formule.clonerPourDuplication();
			}
			if (clone instanceof Pour) {
				Pour timbre = (Pour) clone;
				//
				if (timbre.acteur != null)
					if (livretFamille.containsKey(timbre.acteur))
						timbre.acteur = (Acteur) livretFamille.get(timbre.acteur);
					else
						timbre.acteur = (Acteur) timbre.acteur.clonerPourDuplication();
				//
				if (timbre.indexDebut != null)
					if (livretFamille.containsKey(timbre.indexDebut))
						timbre.indexDebut = (Acteur) livretFamille.get(timbre.indexDebut);
					else
						timbre.indexDebut = (Acteur) timbre.indexDebut.clonerPourDuplication();
				//
				if (timbre.indexFin != null)
					if (livretFamille.containsKey(timbre.indexFin))
						timbre.indexFin = (Acteur) livretFamille.get(timbre.indexFin);
					else
						timbre.indexFin = (Acteur) timbre.indexFin.clonerPourDuplication();
			}
			if (clone instanceof TantQue) {
				TantQue timbre = (TantQue) clone;
				if (timbre.formule != null)
					if (livretFamille.containsKey(timbre.formule))
						timbre.formule = (Acteur) livretFamille.get(timbre.formule);
					else
						timbre.formule = (Acteur) timbre.formule.clonerPourDuplication();
			}
			if (clone instanceof Timbre) {
				Timbre timbre = (Timbre) clone;
				if (timbre.timbreSuivant != null)
					timbre.timbreSuivant = (Timbre) livretFamille.get(timbre.timbreSuivant);
			}
			if (clone instanceof Verbe) {
				Verbe timbre = (Verbe) clone;
				for (int i = 0; i < timbre.nbParametres; i++) {
					if (timbre.parametres[i] != null) {
						if (livretFamille.containsKey(timbre.parametres[i]))
							timbre.parametres[i] = (Acteur) livretFamille.get(timbre.parametres[i]);
						else
							timbre.parametres[i] = (Acteur) timbre.parametres[i].clonerPourDuplication();
					}
				}
			}
		}
		return clones;
	}

}
