package org.linotte.frame.cahier.timbre.outils;

import java.util.ArrayList;
import java.util.List;

import org.linotte.frame.cahier.timbre.JPanelPlancheATimbre;
import org.linotte.frame.cahier.timbre.entite.Acteur;
import org.linotte.frame.cahier.timbre.entite.BlocDebut;
import org.linotte.frame.cahier.timbre.entite.BlocFin;
import org.linotte.frame.cahier.timbre.entite.Bouton;
import org.linotte.frame.cahier.timbre.entite.Commentaire;
import org.linotte.frame.cahier.timbre.entite.Condition;
import org.linotte.frame.cahier.timbre.entite.Pour;
import org.linotte.frame.cahier.timbre.entite.TantQue;
import org.linotte.frame.cahier.timbre.entite.Timbre;
import org.linotte.frame.cahier.timbre.entite.Verbe;
import org.linotte.frame.cahier.timbre.entite.e.Parfum;
import org.linotte.frame.cahier.timbre.entite.e.Role;
import org.linotte.frame.cahier.timbre.ui.Planche;

/**
 * 
 * Classe manipulant les timbres 
 * 
 * @author CPC
 *
 */
public class TimbresHelper {

	/**
	 * @param boutons_fixes 
	 * @param planche 
	 * 
	 */
	public static void constructionBoutons(List<Timbre> boutons_fixes, Planche planche, JPanelPlancheATimbre panel) {
		boutons_fixes.add(new Bouton("Variable texte", new Acteur(null, Role.TEXTE, null, planche), planche));
		boutons_fixes.add(new Bouton("Variable nombre", new Acteur(null, Role.NOMBRE, null, planche), planche));
		boutons_fixes.add(new Bouton("Demander", new Verbe(Parfum.DEMANDER, "Demander", 1, planche), planche));
		boutons_fixes.add(new Bouton("Afficher", new Verbe(Parfum.AFFICHER, "Afficher", 1, planche), planche));
		boutons_fixes.add(new Bouton("Valoir", new Verbe(Parfum.VALOIR, "Valoir", 2, planche), planche));
		boutons_fixes.add(new Bouton("Efface tableau", new Verbe(Parfum.EFFACERTABLEAU, "Effacer tableau", 0, planche), planche));
		boutons_fixes.add(new Bouton("Condition", new Condition(planche), planche));
		boutons_fixes.add(new Bouton("Tant que", new TantQue(planche), planche));
		boutons_fixes.add(new Bouton("Pour", new Pour(planche), planche));
		boutons_fixes.add(new Bouton("Commentaire", new Commentaire(planche), planche));
		boutons_fixes.add(new Bouton("Annuler action", Parfum.UNDO, planche, panel));

		int y = 10;
		for (Timbre timbre : boutons_fixes) {
			timbre.x = 10;
			timbre.y = y;
			y = y + 35;
		}

	}

	/**
	 * @param moi
	 */
	public static void supprimerTimbre(final Timbre moi) {

		//((Planche) moi.planche).historique.ajouterEntree();

		// Je me supprime de l'affichage :
		((Planche) moi.planche).timbres.remove(moi);
		// Si c'est une condition :
		if (moi instanceof BlocDebut) {
			//TODO c'est assez moche... et pas optimisé. A revoir
			BlocFin blocfin = supprimerTimbreCascadeBloc(moi.timbreSuivant);
			if (moi instanceof Condition)
				supprimerTimbreCascadeBloc(((Condition) moi).timbreSuivantSecondaire);
			moi.timbreSuivant = blocfin.timbreSuivant;
			supprimerTimbre(blocfin);

		}
		// Je me supprime en tant que suivant :
		for (Timbre timbre : ((Planche) moi.planche).timbres) {
			if (timbre.timbreSuivant == moi) {
				timbre.timbreSuivant = moi.timbreSuivant;
			}
			if (timbre instanceof Condition) {
				Condition condition = (Condition) timbre;
				if (condition.timbreSuivantSecondaire == moi) {
					condition.timbreSuivantSecondaire = moi.timbreSuivant;
				}
			}
		}
	}

	private static BlocFin supprimerTimbreCascadeBloc(final Timbre timbre) {
		Timbre suivant = timbre;
		while (!(suivant instanceof BlocFin)) {
			supprimerTimbre(suivant);
			suivant = suivant.timbreSuivant;
		}
		return (BlocFin) suivant;

	}

	/**
	 * Cette méthode est appelée quand on attache un timbre à un autre timbre à la fin d'un glissé-collé.
	 * @param source
	 * @param destination
	 */
	public static void deplacerUnTimbre(final Timbre source, final Timbre destination) {
		if (!(destination.timbreSuivant == source) && !(destination instanceof BlocFin && ((BlocFin) destination).blocDebut == source)) {

			//((Planche) source.planche).historique.ajouterEntree();

			// Pas déjà attaché au même timbre

			Timbre ancien = null; // D
			if (destination.timbreSuivant != null) {
				// Timbre qui sera remplacé
				ancien = destination.timbreSuivant; // B
			}

			// Vérification si je ne suis pas le premier d'une série :
			for (Timbre timbre : ((Planche) source.planche).timbres) {
				if (timbre.timbreSuivant == source) {
					// Je ne suis pas premier !
					// [A] -> [source] -> [C] devient [A] -> [C]
					timbre.timbreSuivant = source.timbreSuivant;
					source.timbreSuivant = null;
					// On decalle tous les timbres à droite !
					recherchePremiertimbre(timbre).getUI().positionnerProchainTimbre();
				}
				// Cas des Conditions :
				if (timbre instanceof Condition && ((Condition) timbre).timbreSuivantSecondaire == source) {
					// Je ne suis pas premier !
					// [A] -> [source] -> [C] devient [A] -> [C]
					((Condition) timbre).timbreSuivantSecondaire = source.timbreSuivant;
					source.timbreSuivant = null;
					// On decalle tous les timbres à droite !
					recherchePremiertimbre(timbre).getUI().positionnerProchainTimbre();
				}
			}

			destination.timbreSuivant = source; // C

			if (ancien != null) { // On décalle ce timbre à la fin des timbres insérés
				Timbre suivant = source.timbreSuivant;
				if (suivant != null) {
					while (suivant.timbreSuivant != null) {
						suivant = suivant.timbreSuivant;
					}
					suivant.timbreSuivant = ancien;
				} else {
					source.timbreSuivant = ancien;
				}
			}

		}

	}

	public static boolean timbreSeul(Timbre moi) {
		for (Timbre timbre : ((Planche) moi.planche).timbres) {
			if (timbre.timbreSuivant == moi) {
				return false;
			}
			if (timbre instanceof Condition && ((Condition) timbre).timbreSuivantSecondaire == moi) {
				return false;
			}
		}
		return true;
	}

	public static Timbre recherchePremiertimbre(Timbre moi) {
		List<Timbre> timbres = timbresPrecedents(moi);
		if (timbres.isEmpty())
			return moi;
		else
			return recherchePremiertimbre(timbres.get(0));
	}

	public static List<Timbre> timbresPrecedents(Timbre moi) {
		List<Timbre> timbres = new ArrayList<Timbre>();
		for (Timbre timbre : ((Planche) moi.planche).timbres) {
			if (timbre.timbreSuivant == moi) {
				timbres.add(timbre);
			}
			if (timbre instanceof Condition && (((Condition) timbre).timbreSuivantSecondaire) == moi) {
				timbres.add(timbre);
			}
		}
		return timbres;
	}

	/**
	 * Cette méthode est appelée quand on attache un timbre SECONDAIRE à un autre timbre (@Condition) à la fin d'un glissé-collé.
	 * @param source
	 * @param destination
	 */
	public static void deplacerUnTimbreSecondaire(final Timbre source, final Condition destination) {

		if (!(destination.timbreSuivantSecondaire == source)) {

			//((Planche) source.planche).historique.ajouterEntree();

			// Pas déjà attaché au même timbre

			Timbre ancien = null; // D
			if (destination.timbreSuivantSecondaire != null) {
				// Timbre qui sera remplacé
				ancien = destination.timbreSuivantSecondaire; // B
			}

			// Vérification si je ne suis pas le premier d'une série :
			for (Timbre timbre : ((Planche) source.planche).timbres) {
				if (timbre.timbreSuivant == source) {
					// Je ne suis pas premier !
					// [A] -> [source] -> [C] devient [A] -> [C]
					timbre.timbreSuivant = source.timbreSuivant;
					source.timbreSuivant = null;
					// On decalle tous les timbres à droite !
					recherchePremiertimbre(timbre).getUI().positionnerProchainTimbre();
				}
				// Cas des Conditions :
				if (timbre instanceof Condition && ((Condition) timbre).timbreSuivantSecondaire == source) {
					// Je ne suis pas premier !
					// [A] -> [source] -> [C] devient [A] -> [C]
					((Condition) timbre).timbreSuivantSecondaire = source.timbreSuivant;
					source.timbreSuivant = null;
					// On decalle tous les timbres à droite !
					recherchePremiertimbre(timbre).getUI().positionnerProchainTimbre();
				}

			}

			destination.timbreSuivantSecondaire = source; // C

			if (ancien != null) { // On décalle ce timbre à la fin des timbres insérés
				Timbre suivant = source.timbreSuivant;
				if (suivant != null) {
					while (suivant.timbreSuivant != null) {
						suivant = suivant.timbreSuivant;
					}
					suivant.timbreSuivant = ancien;
				} else {
					source.timbreSuivant = ancien;
				}
			}

		}
	}

}
