package org.linotte.frame.cahier.timbre.ui.theme.simple;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.math.BigDecimal;

import javax.swing.JOptionPane;

import org.linotte.frame.cahier.timbre.entite.Acteur;
import org.linotte.frame.cahier.timbre.entite.Textuel;
import org.linotte.frame.cahier.timbre.entite.Timbre;
import org.linotte.frame.cahier.timbre.entite.e.Role;
import org.linotte.frame.cahier.timbre.outils.TimbresHelper;
import org.linotte.frame.cahier.timbre.ui.Planche;
import org.linotte.frame.cahier.timbre.ui.i.ActeurUI;
import org.linotte.frame.cahier.timbre.ui.i.PlancheUI;
import org.linotte.moteur.xml.analyse.Mathematiques;

public class ActeurSimpleUI implements ActeurUI {

	public Acteur moi;

	private int decallage_y;

	protected ActeurSimpleUI(Acteur acteur) {
		moi = acteur;
		moi.largeur = Abaque.LARGEUR_TIMBRE;
		moi.hauteur = Abaque.HAUTEUR_TIMBRE + 10;
	}

	@Override
	public void pre_dessine(PlancheUI planche) {

		decallage_y = -moi.hauteur / 2 + 20;

		((TimbreSimpleUI) ((Timbre) moi).ui).decallage_y = decallage_y;
		((TextuelSimpleUI) ((Textuel) moi).ui).decallage_y = decallage_y;

	}

	@Override
	public void dessine(PlancheUI planche) {

		Graphics2D g2d = ((Planche) planche).graphics;

		Color couleur = null;

		switch (moi.role) {
		case TEXTE:
			couleur = Palette.ACTEUR_TEXTE;
			break;
		case NOMBRE:
			couleur = Palette.ACTEUR_NOMBRE;
			break;
		case FORMULE:
			couleur = Palette.ACTEUR_FORMULE;
			break;
		}

		Composite old = g2d.getComposite();
		if (moi.glisser) {
			g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
		}

		if (moi.survol && moi.zoneActive == null) {
			g2d.setColor(couleur.brighter());
		} else
			g2d.setColor(couleur);

		// Timbre :
		g2d.fillRoundRect(moi.x, moi.y + decallage_y, moi.largeur, moi.hauteur, Abaque.ANGLE_TIMBRE, Abaque.ANGLE_TIMBRE);

		// Attache avant :
		Polygon triangle = new Polygon(
				new int[] { //
						moi.x + moi.largeur, // A 
						moi.x + moi.largeur, // B
						moi.x + Abaque.ATTACHE_AVANT_TIMBRE / 2 + moi.largeur }, // C
				new int[] { // 
						moi.y + decallage_y + moi.hauteur / 2 - Abaque.ATTACHE_AVANT_TIMBRE / 2, // A
						moi.y + decallage_y + moi.hauteur / 2 + Abaque.ATTACHE_AVANT_TIMBRE / 2, // B
						moi.y + decallage_y + moi.hauteur / 2// C
		}, 3);
		g2d.fillPolygon(triangle);

		// Rôle :
		g2d.setColor(Palette.ROLE);
		g2d.drawString(moi.role.name(), moi.x + 10, moi.y + moi.hauteur - 6 + decallage_y);
		// Valeur :
		g2d.setColor(Palette.VALEUR);
		if (moi.role == Role.TEXTE)
			g2d.drawString("\"" + (moi.valeur == null ? "" : moi.valeur) + "\"", moi.x + 10, moi.y + moi.hauteur / 2 + 7 + decallage_y);
		else
			g2d.drawString(moi.valeur == null ? "0" : moi.valeur, moi.x + 10, moi.y + moi.hauteur / 2 + 7 + decallage_y);

		g2d.setComposite(old);
	}

	@Override
	public boolean clique(int x, int y) {
		{
			// Clique sur le nom
			Shape shape = new Rectangle2D.Double(moi.x + 5, moi.y + 5 + decallage_y, moi.largeur - 6, 25);
			boolean clique = shape.contains(x, y);
			if (clique) {
				// Acteur anonyme texte :
				String valeur = (String) JOptionPane.showInputDialog(null, "Nom de la variable ?", "Modification d'une variable", JOptionPane.QUESTION_MESSAGE,
						null, null, null);
				if (valeur != null)
					moi.texte = valeur;
				// La largeur du texte peut modifier la largeur du timbre et décaller le prochaine timbre :
				TimbresHelper.recherchePremiertimbre(moi).timbreSuivant.getUI().positionnerProchainTimbre();
				return true;
			}
		}
		{
			// Clique sur la valeur
			Shape shape = new Rectangle2D.Double(moi.x + 10, moi.y + moi.hauteur / 2 + decallage_y, moi.largeur - 6, 25);
			boolean clique = shape.contains(x, y);
			if (clique) {
				// Acteur anonyme texte :
				String valeur = (String) JOptionPane.showInputDialog(null, "Valeur de la variable ?", "Modification d'une variable",
						JOptionPane.QUESTION_MESSAGE, null, null, null);
				if (valeur != null) {
					if (moi.role == Role.NOMBRE) {
						BigDecimal r = Mathematiques.isBigDecimal(valeur);
						if (r != null) {
							moi.valeur = valeur;
						} else {
							JOptionPane.showMessageDialog(null, "Valeur incorrecte !");
						}
					} else {
						moi.valeur = valeur;
					}
				}
				// La largeur du texte peut modifier la largeur du timbre et décaller le prochaine timbre :
				TimbresHelper.recherchePremiertimbre(moi).timbreSuivant.getUI().positionnerProchainTimbre();
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean contient(int x, int y) {
		Shape shape = new Rectangle2D.Double(moi.x + 2, moi.y + 30 + decallage_y, moi.largeur - 6, 25);
		boolean b = shape.contains(x, y);
		return b;
	}

	@Override
	public void positionnerProchainTimbre() {

	}

}
