package org.linotte.frame.cahier.timbre.ui.theme.simple;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.math.BigDecimal;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JOptionPane;

import org.linotte.frame.cahier.timbre.entite.Acteur;
import org.linotte.frame.cahier.timbre.entite.BlocFin;
import org.linotte.frame.cahier.timbre.entite.Pour;
import org.linotte.frame.cahier.timbre.entite.Timbre;
import org.linotte.frame.cahier.timbre.entite.e.Role;
import org.linotte.frame.cahier.timbre.entite.e.ZoneActive;
import org.linotte.frame.cahier.timbre.outils.TimbresHelper;
import org.linotte.frame.cahier.timbre.ui.Planche;
import org.linotte.frame.cahier.timbre.ui.i.PlancheUI;
import org.linotte.frame.cahier.timbre.ui.i.PourUI;
import org.linotte.moteur.xml.analyse.Mathematiques;

public class PourSimpleUI implements PourUI {

	public Pour moi;

	public Color couleur = Palette.CONDITION;

	private int decallage_y;

	private final int HAUTEUR_BOITE = 25;

	private final int DISTANCE_ACTEURS = 60;

	protected PourSimpleUI(Pour pour) {
		moi = pour;
	}

	@Override
	public void pre_dessine(PlancheUI plancheUI) {

		int index = calculerHauteur(moi.timbreSuivant);
		moi.hauteur = (int) (Abaque.HAUTEUR_TIMBRE * (2 + (1 + index)));

		BlocFinSimpleUI blocFinSimpleUI = (BlocFinSimpleUI) moi.blocfin.ui;
		blocFinSimpleUI.hauteurCalculee = moi.hauteur;
		blocFinSimpleUI.moi.hauteur = moi.hauteur;

		decallage_y = -40 - (int) (Abaque.HAUTEUR_TIMBRE / 2 * (1 + index));

		((TimbreSimpleUI) ((Timbre) moi).ui).decallage_y = decallage_y;

		Planche planche = ((Planche) plancheUI);
		Graphics2D g2d = planche.graphics;

		// On précalcule la largeur on fonction du texte de la condition :
		// En premier, il faut calculer la largeur :
		int largeurMax = 0;
		if (moi.acteur != null && moi.acteur.valeur != null) {
			largeurMax = g2d.getFontMetrics().stringWidth(moi.acteur.valeur);
		}
		if (largeurMax > Abaque.LARGEUR_TIMBRE - 10) {
			moi.largeur = 30 + largeurMax;
		} else {
			moi.largeur = Abaque.LARGEUR_TIMBRE;
		}

		Composite old = g2d.getComposite();

		if (moi.glisser)
			g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));

		// Attachement :
		if (moi.timbreSuivant != null && !(moi.glisser && (moi.timbreSuivant instanceof BlocFin) && moi.timbreSuivant.timbreSuivant == null)) {
			g2d.setColor(Palette.ATTACHEMENT);
			g2d.setStroke(new BasicStroke(3));
			if (moi.glisser) {
				g2d.drawLine(moi.init_x + moi.largeur, moi.init_y + 20, moi.timbreSuivant.x, moi.init_y + 20);
			} else {
				if (moi.timbreSuivant.glisser) {
					g2d.drawLine(moi.x + moi.largeur, moi.y + 20, moi.timbreSuivant.init_x, moi.y + 20);
				} else {
					g2d.drawLine(moi.x + moi.largeur, moi.y + 20, moi.timbreSuivant.x, moi.y + 20);
				}
			}
			g2d.setStroke(new BasicStroke());

		}
		g2d.setComposite(old);

	}

	@Override
	public void dessine(PlancheUI plancheUI) {

		Planche planche = ((Planche) plancheUI);
		Graphics2D g2d = planche.graphics;

		Composite old = g2d.getComposite();
		if (moi.glisser)
			g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));

		if (moi.survol && moi.zoneActive == null)
			g2d.setColor(couleur.brighter());
		else if (moi.blocfin.survol)
			g2d.setColor(couleur.darker().darker());
		else
			g2d.setColor(couleur);

		// Timbre :
		g2d.fillRoundRect(moi.x, moi.y + decallage_y, moi.largeur, moi.hauteur, 50, 50);

		// Attache avant :
		Polygon triangle = new Polygon(
				new int[] { //
						moi.x + moi.largeur, // A 
						moi.x + moi.largeur, // B
						moi.x + Abaque.ATTACHE_AVANT_TIMBRE / 2 + moi.largeur }, // C
				new int[] { // 
						moi.y + decallage_y + moi.hauteur / 2 - Abaque.ATTACHE_AVANT_TIMBRE / 2, // A
						moi.y + decallage_y + moi.hauteur / 2 + Abaque.ATTACHE_AVANT_TIMBRE / 2, // B
						moi.y + decallage_y + moi.hauteur / 2// C
		}, 3);
		g2d.fillPolygon(triangle);

		g2d.setColor(Palette.TEXTE);
		g2d.drawString("Pour", moi.x + moi.largeur / 3, moi.y - moi.hauteur / 2 + 10);

		int hauteurActeur;

		// Zone ACTEUR de la boucle FOR :
		// ******************************
		hauteurActeur = 10 + decallage_y;
		afficheActeur(hauteurActeur, g2d, moi.acteur, moi.zoneActive, ZoneActive.POUR_ACTEUR, null);

		// Zone INDEX DEBUT de la boucle FOR :
		// ***********************************
		hauteurActeur = 10 + DISTANCE_ACTEURS + decallage_y;
		afficheActeur(hauteurActeur, g2d, moi.indexDebut, moi.zoneActive, ZoneActive.POUR_INDEX_DEBUT, "allant de");

		// Zone INDEX FIN de la boucle FOR :
		// *********************************
		hauteurActeur = 10 + DISTANCE_ACTEURS * 2 + decallage_y;
		afficheActeur(hauteurActeur, g2d, moi.indexFin, moi.zoneActive, ZoneActive.POUR_INDEX_FIN, "à");

		g2d.setComposite(old);
	}

	/**
	 * @param hauteurActeur 
	 * @param g2d
	 * @param pourActeur 
	 * @param acteur 
	 * @param zoneCible 
	 * @param texte 
	 */
	private void afficheActeur(int hauteurActeur, Graphics2D g2d, Acteur acteur, ZoneActive zone, ZoneActive zoneCible, String texte) {
		if (zoneCible == zone) {
			g2d.setStroke(new BasicStroke(2));
			g2d.setColor(Palette.HALO_PARAMETRE_ACTIF);
			g2d.drawRect(moi.x + 8, moi.y + hauteurActeur - 2, moi.largeur - 21, HAUTEUR_BOITE + 4);
			g2d.setStroke(new BasicStroke());
		}

		if (texte != null) {
			g2d.setColor(Palette.TEXTE);
			g2d.drawString(texte, moi.x + 12, moi.y + hauteurActeur + 15 - 30);
		}

		if (acteur == null) {
			g2d.setColor(Palette.TEXTE);
			g2d.drawRect(moi.x + 10, moi.y + hauteurActeur, moi.largeur - 25, HAUTEUR_BOITE);
			g2d.drawString("?", moi.x + 12, moi.y + hauteurActeur + 15);
		} else {
			if (acteur.texte == null) {
				g2d.setColor(Palette.HALO_PARAMETRE);
				g2d.drawRect(moi.x + 10, moi.y + hauteurActeur, moi.largeur - 25, HAUTEUR_BOITE);
				//Acteur anonyme
				g2d.setColor(Palette.TEXTE);
				if (acteur.role == Role.TEXTE)
					g2d.drawString("\"" + acteur.valeur + "\"", moi.x + 12, moi.y + hauteurActeur + 15);
				else
					g2d.drawString(acteur.valeur, moi.x + 12, moi.y + hauteurActeur + 15);
			} else {
				g2d.setColor(Palette.ACTEUR);
				g2d.fillRect(moi.x + 10, moi.y + hauteurActeur, moi.largeur - 25, HAUTEUR_BOITE);
				g2d.setColor(Palette.HALO_PARAMETRE);
				g2d.drawRect(moi.x + 10, moi.y + hauteurActeur, moi.largeur - 25, HAUTEUR_BOITE);
				g2d.setColor(Palette.TEXTE);
				g2d.drawString(acteur.texte, moi.x + 12, moi.y + hauteurActeur + 15);
			}
		}
	}

	@Override
	public boolean clique(int x, int y) {
		// Clique acteur
		int hauteurActeur;
		{
			hauteurActeur = 10 + decallage_y;
			Shape shape = new Rectangle2D.Double(moi.x + 10, moi.y + hauteurActeur, moi.largeur - 25, HAUTEUR_BOITE);
			boolean clique = shape.contains(x, y);
			if (clique) {
				moi.acteur = demanderValeur(moi.acteur);
				// La largeur du texte peut modifier la largeur du timbre et décaller le prochaine timbre :
				TimbresHelper.recherchePremiertimbre(moi).timbreSuivant.getUI().positionnerProchainTimbre();
				return true;
			}
		}
		// Clique index_debut
		{
			hauteurActeur = 10 + DISTANCE_ACTEURS + decallage_y;
			Shape shape = new Rectangle2D.Double(moi.x + 10, moi.y + hauteurActeur, moi.largeur - 25, HAUTEUR_BOITE);
			boolean clique = shape.contains(x, y);
			if (clique) {
				moi.indexDebut = demanderValeur(moi.indexDebut);
				// La largeur du texte peut modifier la largeur du timbre et décaller le prochaine timbre :
				TimbresHelper.recherchePremiertimbre(moi).timbreSuivant.getUI().positionnerProchainTimbre();
				return true;
			}
		}
		// Clique index_fin
		{
			hauteurActeur = 10 + DISTANCE_ACTEURS * 2 + decallage_y;
			Shape shape = new Rectangle2D.Double(moi.x + 10, moi.y + hauteurActeur, moi.largeur - 25, HAUTEUR_BOITE);
			boolean clique = shape.contains(x, y);
			if (clique) {
				moi.indexFin = demanderValeur(moi.indexFin);
				// La largeur du texte peut modifier la largeur du timbre et décaller le prochaine timbre :
				TimbresHelper.recherchePremiertimbre(moi).timbreSuivant.getUI().positionnerProchainTimbre();
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean contient(int x, int y) {
		return false;
	}

	@Override
	public void positionnerProchainTimbre() {

		moi.pre_dessine(); // On force les calcules.

		if (moi.timbreSuivant != null) {

			moi.timbreSuivant.x = moi.x + moi.largeur + 10;
			moi.timbreSuivant.y = moi.y;

			if (moi.timbreSuivant.timbreSuivant != null)
				moi.timbreSuivant.getUI().positionnerProchainTimbre();

			if (moi.timbreSuivant instanceof BlocFin) {
				((BlocFinSimpleUI) ((BlocFin) moi.timbreSuivant).ui).visible = true;
				return;
			}

		}

		// Les blocs ont été décallés, on s'occupe du blocFin :
		BlocFin blocfin = moi.blocfin;
		List<Timbre> precedents = TimbresHelper.timbresPrecedents(blocfin);
		if (!precedents.isEmpty()) {
			// On prend celui le plus à droite :
			Timbre plusADroite = precedents.get(0);
			for (Timbre timbre : precedents) {
				if ((timbre.x + timbre.largeur) > (plusADroite.x + plusADroite.largeur)) {
					plusADroite = timbre;
				}
			}
			blocfin.x = plusADroite.x + plusADroite.largeur + 10;
			blocfin.y = moi.y;
		}
		blocfin.getUI().positionnerProchainTimbre();
	}

	@Override
	public ZoneActive accepteZoneParametres(Acteur acteur) {
		int hauteurActeur;
		// On reduit la zone de contact pour faciliter l'attachement au précédent timbre
		int largeur = moi.largeur - 25;
		{
			hauteurActeur = 10 + decallage_y;
			Shape shape = new Rectangle2D.Double(moi.x + 10 + largeur, moi.y + hauteurActeur, largeur, HAUTEUR_BOITE);
			boolean b = shape.intersects((Rectangle2D) ((Timbre) acteur).ui.forme());
			if (b) {
				return ZoneActive.POUR_ACTEUR;
			}
		}
		{
			hauteurActeur = 10 + DISTANCE_ACTEURS + decallage_y;
			Shape shape = new Rectangle2D.Double(moi.x + 10 + largeur, moi.y + hauteurActeur, largeur, HAUTEUR_BOITE);
			boolean b = shape.intersects((Rectangle2D) ((Timbre) acteur).ui.forme());
			if (b) {
				return ZoneActive.POUR_INDEX_DEBUT;
			}
		}
		{
			hauteurActeur = 10 + DISTANCE_ACTEURS * 2 + decallage_y;
			Shape shape = new Rectangle2D.Double(moi.x + 10 + largeur, moi.y + hauteurActeur, largeur, HAUTEUR_BOITE);
			boolean b = shape.intersects((Rectangle2D) ((Timbre) acteur).ui.forme());
			if (b) {
				return ZoneActive.POUR_INDEX_FIN;
			}
		}
		return null;
	}

	/*
	 * Méthodes privées :
	 */

	/**
	 * @return 
	 * 
	 */
	private Acteur demanderValeur(Acteur acteur) {
		final Role[] role = { Role.NOMBRE, Role.TEXTE, Role.FORMULE };
		final JComboBox roles = new JComboBox<Role>(role);
		String initValeur = null;
		if (acteur != null && acteur.texte == null) {
			roles.setSelectedItem(acteur.role);
			initValeur = acteur.valeur;
		}
		final Object msg[] = { "Valeur de la variable ?", roles };
		// Acteur anonyme texte :
		String valeur = (String) JOptionPane.showInputDialog(null, msg, "Création d'une variable anonyme", JOptionPane.QUESTION_MESSAGE, null, null,
				initValeur);
		if (valeur != null) {
			boolean correcte = true;

			if ((Role) roles.getSelectedItem() == Role.NOMBRE) {
				BigDecimal r = Mathematiques.isBigDecimal(valeur);
				if (r == null) {
					JOptionPane.showMessageDialog(null, "Valeur incorrecte !");
					correcte = false;
				}
			}

			if (correcte) {
				acteur = new Acteur(null, (Role) roles.getSelectedItem(), valeur, null);
				//TODO, à changer :
				acteur.texte = null;
			}
		}
		return acteur;
	}

	private int calculerHauteur(Timbre suivant) {
		if (suivant == null || (suivant instanceof BlocFin && ((BlocFin) suivant).blocDebut instanceof Pour)) // <- Que des "Pour"
			return 0;
		else if (suivant instanceof Pour)
			return 1 + calculerHauteur(suivant.timbreSuivant);
		else
			return 0 + calculerHauteur(suivant.timbreSuivant);
	}

}
