package org.linotte.frame.cahier.timbre.ui.theme.simple;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JOptionPane;

import org.linotte.frame.cahier.timbre.entite.Acteur;
import org.linotte.frame.cahier.timbre.entite.BlocFin;
import org.linotte.frame.cahier.timbre.entite.TantQue;
import org.linotte.frame.cahier.timbre.entite.Timbre;
import org.linotte.frame.cahier.timbre.entite.e.Role;
import org.linotte.frame.cahier.timbre.outils.TimbresHelper;
import org.linotte.frame.cahier.timbre.ui.Planche;
import org.linotte.frame.cahier.timbre.ui.i.PlancheUI;
import org.linotte.frame.cahier.timbre.ui.i.TantQueUI;

public class TantQueSimpleUI implements TantQueUI {

	public TantQue moi;

	public Color couleur = Palette.CONDITION;

	private int decallage_y;

	protected TantQueSimpleUI(TantQue tantque) {
		moi = tantque;
	}

	@Override
	public void pre_dessine(PlancheUI plancheUI) {

		int index = calculerHauteur(moi.timbreSuivant);
		moi.hauteur = (int) (Abaque.HAUTEUR_TIMBRE * (1.5 + index));

		BlocFinSimpleUI blocFinSimpleUI = (BlocFinSimpleUI) moi.blocfin.ui;
		blocFinSimpleUI.hauteurCalculee = moi.hauteur;
		blocFinSimpleUI.moi.hauteur = moi.hauteur;

		decallage_y = -25 - (int) (Abaque.HAUTEUR_TIMBRE / 2 * (index));

		((TimbreSimpleUI) ((Timbre) moi).ui).decallage_y = decallage_y;

		Planche planche = ((Planche) plancheUI);
		Graphics2D g2d = planche.graphics;

		// On précalcule la largeur on fonction du texte de la condition :
		// En premier, il faut calculer la largeur :
		int largeurMax = 0;
		if (moi.formule != null) {
			largeurMax = g2d.getFontMetrics().stringWidth(moi.formule.valeur);
		}
		if (largeurMax > Abaque.LARGEUR_TIMBRE - 10) {
			moi.largeur = 30 + largeurMax;
		} else {
			moi.largeur = Abaque.LARGEUR_TIMBRE;
		}

		Composite old = g2d.getComposite();

		if (moi.glisser)
			g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));

		// Attachement :
		if (moi.timbreSuivant != null && !(moi.glisser && (moi.timbreSuivant instanceof BlocFin) && moi.timbreSuivant.timbreSuivant == null)) {
			g2d.setColor(Palette.ATTACHEMENT);
			g2d.setStroke(new BasicStroke(3));
			if (moi.glisser) {
				g2d.drawLine(moi.init_x + moi.largeur, moi.init_y + 20, moi.timbreSuivant.x, moi.init_y + 20);
			} else {
				if (moi.timbreSuivant.glisser) {
					g2d.drawLine(moi.x + moi.largeur, moi.y + 20, moi.timbreSuivant.init_x, moi.y + 20);
				} else {
					g2d.drawLine(moi.x + moi.largeur, moi.y + 20, moi.timbreSuivant.x, moi.y + 20);
				}
			}
			g2d.setStroke(new BasicStroke());

		}
		g2d.setComposite(old);

	}

	@Override
	public void dessine(PlancheUI plancheUI) {

		Planche planche = ((Planche) plancheUI);
		Graphics2D g2d = planche.graphics;

		Composite old = g2d.getComposite();
		if (moi.glisser)
			g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));

		if (moi.survol && moi.zoneActive == null)
			g2d.setColor(couleur.brighter());
		else if (moi.blocfin.survol)
			g2d.setColor(couleur.darker().darker());
		else
			g2d.setColor(couleur);

		// Timbre :
		g2d.fillOval(moi.x, moi.y + decallage_y, moi.largeur, moi.hauteur);

		// Attache avant :
		Polygon triangle = new Polygon(
				new int[] { //
						moi.x + moi.largeur, // A 
						moi.x + moi.largeur, // B
						moi.x + Abaque.ATTACHE_AVANT_TIMBRE / 2 + moi.largeur }, // C
				new int[] { // 
						moi.y + decallage_y + moi.hauteur / 2 - Abaque.ATTACHE_AVANT_TIMBRE / 2, // A
						moi.y + decallage_y + moi.hauteur / 2 + Abaque.ATTACHE_AVANT_TIMBRE / 2, // B
						moi.y + decallage_y + moi.hauteur / 2// C
		}, 3);
		g2d.fillPolygon(triangle);

		g2d.setColor(Palette.TEXTE);
		g2d.drawString("Tant que", moi.x + moi.largeur / 4, moi.y - moi.hauteur / 2 + 10);

		// Zone formule de la condition :
		g2d.setColor(Palette.HALO_PARAMETRE);
		int hauteur_boite = 25;
		g2d.drawRect(moi.x + 10, moi.y + (hauteur_boite - 10) / 2, moi.largeur - 25, hauteur_boite);
		if (moi.formule == null) {
			g2d.setColor(Palette.TEXTE);
			g2d.drawString("?", moi.x + 12, moi.y + (hauteur_boite - 10) / 2 + 15);
		} else {
			g2d.setColor(Palette.TEXTE);
			g2d.drawString(moi.formule.valeur, moi.x + 22, moi.y + (hauteur_boite - 10) / 2 + 15);
		}

		g2d.setComposite(old);
	}

	@Override
	public boolean clique(int x, int y) {
		// Clique sur la formule
		{
			int hauteur_boite = 25;
			Shape shape = new Rectangle2D.Double(moi.x + 10, moi.y + (hauteur_boite - 10) / 2, moi.largeur - 25, hauteur_boite);
			boolean clique = shape.contains(x, y);
			if (clique) {
				demanderValeur();
				// La largeur du texte peut modifier la largeur du timbre et décaller le prochaine timbre :
				TimbresHelper.recherchePremiertimbre(moi).timbreSuivant.getUI().positionnerProchainTimbre();
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean contient(int x, int y) {
		Shape shape = new Rectangle2D.Double(moi.x + 2, moi.y + 30, moi.largeur - 6, 25);
		boolean b = shape.contains(x, y);
		return b;
	}

	@Override
	public void positionnerProchainTimbre() {

		moi.pre_dessine(); // On force les calcules.

		if (moi.timbreSuivant != null) {

			moi.timbreSuivant.x = moi.x + moi.largeur + 10;
			moi.timbreSuivant.y = moi.y;

			if (moi.timbreSuivant.timbreSuivant != null)
				moi.timbreSuivant.getUI().positionnerProchainTimbre();

			if (moi.timbreSuivant instanceof BlocFin) {
				((BlocFinSimpleUI) ((BlocFin) moi.timbreSuivant).ui).visible = true;
				return;
			}

		}

		// Les blocs ont été décallés, on s'occupe du blocFin :
		BlocFin blocfin = moi.blocfin;
		List<Timbre> precedents = TimbresHelper.timbresPrecedents(blocfin);
		if (!precedents.isEmpty()) {
			// On prend celui le plus à droite :
			Timbre plusADroite = precedents.get(0);
			for (Timbre timbre : precedents) {
				if ((timbre.x + timbre.largeur) > (plusADroite.x + plusADroite.largeur)) {
					plusADroite = timbre;
				}
			}
			blocfin.x = plusADroite.x + plusADroite.largeur + 10;
			blocfin.y = moi.y;
		}
		blocfin.getUI().positionnerProchainTimbre();
	}

	/**
	 * 
	 */
	private void demanderValeur() {
		final Role[] role = { Role.FORMULE };
		final JComboBox roles = new JComboBox<Role>(role);
		String initValeur = (moi.formule != null) ? moi.formule.valeur : null;
		final Object msg[] = { "condition :", roles };
		// Acteur anonyme texte :
		String valeur = (String) JOptionPane.showInputDialog(null, msg, "Modification de la condition", JOptionPane.QUESTION_MESSAGE, null, null, initValeur);
		if (valeur != null) {
			moi.formule = new Acteur(null, (Role) roles.getSelectedItem(), valeur, null);
			//TODO, à changer :
			moi.formule.texte = null;
		}
	}

	private int calculerHauteur(Timbre suivant) {
		if (suivant == null || (suivant instanceof BlocFin && ((BlocFin) suivant).blocDebut instanceof TantQue)) // <- Que des Tant que
			return 0;
		else if (suivant instanceof TantQue)
			return 1 + calculerHauteur(suivant.timbreSuivant);
		else
			return 0 + calculerHauteur(suivant.timbreSuivant);
	}

}
