package org.linotte.frame.cahier.timbre.ui.theme.simple;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.math.BigDecimal;

import javax.swing.JComboBox;
import javax.swing.JOptionPane;

import org.linotte.frame.cahier.timbre.entite.Acteur;
import org.linotte.frame.cahier.timbre.entite.Textuel;
import org.linotte.frame.cahier.timbre.entite.Timbre;
import org.linotte.frame.cahier.timbre.entite.Verbe;
import org.linotte.frame.cahier.timbre.entite.e.Role;
import org.linotte.frame.cahier.timbre.entite.e.ZoneActive;
import org.linotte.frame.cahier.timbre.outils.TimbresHelper;
import org.linotte.frame.cahier.timbre.ui.Planche;
import org.linotte.frame.cahier.timbre.ui.i.PlancheUI;
import org.linotte.frame.cahier.timbre.ui.i.VerbeUI;
import org.linotte.moteur.xml.analyse.Mathematiques;

public class VerbeSimpleUI implements VerbeUI {

	public Verbe moi;

	public Color couleur = Palette.VERBE;

	private static final int DECALAGE_PARAMETRE = 35;

	private int decallage_y;

	protected VerbeSimpleUI(Verbe verbe) {
		moi = verbe;
	}

	@Override
	public void pre_dessine(PlancheUI planche) {
		Graphics2D g2d = ((Planche) planche).graphics;

		moi.largeur = Abaque.LARGEUR_TIMBRE;
		moi.hauteur = Abaque.HAUTEUR_TIMBRE - (moi.nbParametres == 0 ? 20 : 0) + (moi.nbParametres * 20);
		decallage_y = -moi.hauteur / 2 + 20;

		((TimbreSimpleUI) ((Timbre) moi).ui).decallage_y = decallage_y;
		((TextuelSimpleUI) ((Textuel) moi).ui).decallage_y = decallage_y;

		// En premier, il faut calculer la largeur :
		if (moi.nbParametres > 0) {
			int largeurMax = 0;
			if (moi.parametres[0] != null) {
				largeurMax = g2d.getFontMetrics().stringWidth("\"" + moi.parametres[0].valeur + "\"");
			}
			if (moi.nbParametres > 1 && moi.parametres[1] != null) {
				int largeurMaxP2 = g2d.getFontMetrics().stringWidth("\"" + moi.parametres[1].valeur + "\"");
				if (largeurMaxP2 > largeurMax)
					largeurMax = largeurMaxP2;
			}
			if (largeurMax > Abaque.LARGEUR_TIMBRE - 10) {
				moi.largeur = Abaque.LARGEUR_TIMBRE + 10 + (largeurMax - Abaque.LARGEUR_TIMBRE);
			} else {
				moi.largeur = Abaque.LARGEUR_TIMBRE;
			}
		}

	}

	@Override
	public void dessine(PlancheUI planche) {

		Graphics2D g2d = ((Planche) planche).graphics;

		Composite old = g2d.getComposite();
		if (moi.glisser) {
			g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
		}

		// Fond du timbre
		if (moi.survol && moi.zoneActive == null)
			g2d.setColor(couleur.brighter());
		else
			g2d.setColor(couleur);

		// Timbre :
		g2d.fillRoundRect(moi.x, moi.y + decallage_y, moi.largeur, moi.hauteur, Abaque.ANGLE_TIMBRE, Abaque.ANGLE_TIMBRE);

		// Attache avant :
		Polygon triangle = new Polygon(
				new int[] { //
						moi.x + moi.largeur, // A 
						moi.x + moi.largeur, // B
						moi.x + Abaque.ATTACHE_AVANT_TIMBRE / 2 + moi.largeur }, // C
				new int[] { // 
						moi.y + decallage_y + moi.hauteur / 2 - Abaque.ATTACHE_AVANT_TIMBRE / 2, // A
						moi.y + decallage_y + moi.hauteur / 2 + Abaque.ATTACHE_AVANT_TIMBRE / 2, // B
						moi.y + decallage_y + moi.hauteur / 2// C
		}, 3);
		g2d.fillPolygon(triangle);

		// Affichage des paramètres :
		if (moi.nbParametres > 0) {

			// Paramètre 1 :

			if (ZoneActive.PARAMETRE_1 == moi.zoneActive) {
				g2d.setStroke(new BasicStroke(2));
				g2d.setColor(Palette.HALO_PARAMETRE_ACTIF);
				g2d.drawRect(moi.x, moi.y + 28 + decallage_y, moi.largeur - 2, 27);
				g2d.setStroke(new BasicStroke());
			}
			if (moi.parametres[0] == null) {
				g2d.setColor(Palette.TEXTE);
				g2d.drawRect(moi.x + 2, moi.y + 30 + decallage_y, moi.largeur - 6, 25);
				g2d.drawString("?", moi.x + 4, moi.y + 45 + decallage_y);
			} else {
				if (moi.parametres[0].texte == null) {
					g2d.setColor(Palette.HALO_PARAMETRE);
					g2d.drawRect(moi.x + 2, moi.y + 30 + decallage_y, moi.largeur - 6, 25);
					//Acteur anonyme
					g2d.setColor(Palette.TEXTE);
					if (moi.parametres[0].role == Role.TEXTE)
						g2d.drawString("\"" + moi.parametres[0].valeur + "\"", moi.x + 4, moi.y + 45 + decallage_y);
					else
						g2d.drawString(moi.parametres[0].valeur, moi.x + 4, moi.y + 45 + decallage_y);
				} else {
					g2d.setColor(Palette.ACTEUR);
					g2d.fillRect(moi.x + 2, moi.y + 30 + decallage_y, moi.largeur - 6, 25);
					g2d.setColor(Palette.HALO_PARAMETRE);
					g2d.drawRect(moi.x + 2, moi.y + 30 + decallage_y, moi.largeur - 6, 25);
					g2d.setColor(Palette.TEXTE);
					g2d.drawString(moi.parametres[0].texte, moi.x + 4, moi.y + 45 + decallage_y);
				}
			}

			// Paramètre 2 :
			if (moi.nbParametres > 1) {
				if (ZoneActive.PARAMETRE_2 == moi.zoneActive) {
					g2d.setStroke(new BasicStroke(2));
					g2d.setColor(Palette.HALO_PARAMETRE_ACTIF);
					g2d.drawRect(moi.x, moi.y + 28 + DECALAGE_PARAMETRE + decallage_y, moi.largeur - 2, 27);
					g2d.setStroke(new BasicStroke());
				}

				if (moi.parametres[1] == null) {
					g2d.setColor(Palette.HALO_PARAMETRE);
					g2d.drawRect(moi.x + 2, moi.y + 30 + DECALAGE_PARAMETRE + decallage_y, moi.largeur - 6, 25);
					g2d.setColor(Palette.TEXTE);
					g2d.drawString("?", moi.x + 4, moi.y + 45 + DECALAGE_PARAMETRE + decallage_y);
				} else {
					if (moi.parametres[1].texte == null) {
						g2d.setColor(Palette.HALO_PARAMETRE);
						g2d.drawRect(moi.x + 2, moi.y + 30 + DECALAGE_PARAMETRE + decallage_y, moi.largeur - 6, 25);
						g2d.setColor(Palette.TEXTE);
						//Acteur anonyme
						if (moi.parametres[1].role == Role.TEXTE)
							g2d.drawString("\"" + moi.parametres[1].valeur + "\"", moi.x + 4, moi.y + 45 + DECALAGE_PARAMETRE + decallage_y);
						else
							g2d.drawString(moi.parametres[1].valeur, moi.x + 4, moi.y + 45 + DECALAGE_PARAMETRE + decallage_y);
					} else {
						g2d.setColor(Palette.ACTEUR);
						g2d.fillRect(moi.x + 2, moi.y + 30 + DECALAGE_PARAMETRE + decallage_y, moi.largeur - 6, 25);
						g2d.setColor(Palette.HALO_PARAMETRE);
						g2d.drawRect(moi.x + 2, moi.y + 30 + DECALAGE_PARAMETRE + decallage_y, moi.largeur - 6, 25);
						g2d.setColor(Palette.TEXTE);
						g2d.drawString(moi.parametres[1].texte, moi.x + 4, moi.y + 45 + DECALAGE_PARAMETRE + decallage_y);
					}
				}
			}

		}

		g2d.setComposite(old);

	}

	@Override
	public boolean clique(int x, int y) {

		if (moi.nbParametres > 0) {

			// Clique sur un paramètre 1 ?
			{
				Shape shape = new Rectangle2D.Double(moi.x + 2, moi.y + 30 + decallage_y, moi.largeur - 6, 25);
				boolean clique = shape.contains(x, y);
				if (clique) {
					demanderValeur(moi.parametres, 0);
					// La largeur du texte peut modifier la largeur du timbre et décaller le prochaine timbre :
					if (TimbresHelper.recherchePremiertimbre(moi).timbreSuivant != null)
						TimbresHelper.recherchePremiertimbre(moi).timbreSuivant.getUI().positionnerProchainTimbre();
					return true;
				}
			}

			// Clique sur un paramètre 2 ?
			{
				Shape shape = new Rectangle2D.Double(moi.x + 2, moi.y + 30 + DECALAGE_PARAMETRE + decallage_y, moi.largeur - 6, 25);
				boolean clique = shape.contains(x, y);
				if (clique) {
					demanderValeur(moi.parametres, 1);
					// La largeur du texte peut modifier la largeur du timbre et décaller le prochaine timbre :
					if (TimbresHelper.recherchePremiertimbre(moi).timbreSuivant != null)
						TimbresHelper.recherchePremiertimbre(moi).timbreSuivant.getUI().positionnerProchainTimbre();
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * 
	 */
	private void demanderValeur(Acteur[] parametres, int position) {
		final Role[] role = { Role.TEXTE, Role.NOMBRE, Role.FORMULE };
		final JComboBox roles = new JComboBox<Role>(role);
		String initValeur = null;
		if (parametres[position] != null && parametres[position].texte == null) {
			roles.setSelectedItem(parametres[position].role);
			initValeur = parametres[position].valeur;
		}
		final Object msg[] = { "Valeur de la variable ?", roles };
		// Acteur anonyme texte :
		String valeur = (String) JOptionPane.showInputDialog(null, msg, "Création d'une variable anonyme", JOptionPane.QUESTION_MESSAGE, null, null,
				initValeur);
		if (valeur != null) {
			boolean correcte = true;

			if ((Role) roles.getSelectedItem() == Role.NOMBRE) {
				BigDecimal r = Mathematiques.isBigDecimal(valeur);
				if (r == null) {
					JOptionPane.showMessageDialog(null, "Valeur incorrecte !");
					correcte = false;
				}
			}

			if (correcte) {
				parametres[position] = new Acteur(null, (Role) roles.getSelectedItem(), valeur, null);
				//TODO, à changer :
				parametres[position].texte = null;
			}
		}
	}

	@Override
	public boolean contient(int x, int y) {
		return false;
	}

	@Override
	public ZoneActive accepteZoneParametres(Acteur acteur) {
		if (moi.nbParametres > 0) {
			// On reduit la zone de contact pour faciliter l'attachement au précédent timbre
			int largeur = (moi.largeur - 6) / 2;
			{
				Shape shape = new Rectangle2D.Double(moi.x + 2 + largeur, moi.y + 30 + decallage_y, largeur, 25);
				boolean b = shape.intersects((Rectangle2D) ((Timbre) acteur).ui.forme());
				if (b) {
					return ZoneActive.PARAMETRE_1;
				}
			}
			{
				if (moi.nbParametres > 1) {
					Shape shape = new Rectangle2D.Double(moi.x + 2 + largeur, moi.y + 30 + DECALAGE_PARAMETRE + decallage_y, largeur, 25);
					boolean b = shape.intersects((Rectangle2D) ((Timbre) acteur).ui.forme());
					if (b) {
						return ZoneActive.PARAMETRE_2;
					}
				}
			}
		}
		return null;
	}

	@Override
	public void positionnerProchainTimbre() {

	}
}
