package org.linotte.moteur.xml.analyse.timbre;

import java.util.List;

import org.linotte.frame.cahier.timbre.entite.Acteur;
import org.linotte.frame.cahier.timbre.entite.BlocDebut;
import org.linotte.frame.cahier.timbre.entite.BlocFin;
import org.linotte.frame.cahier.timbre.entite.Commentaire;
import org.linotte.frame.cahier.timbre.entite.Condition;
import org.linotte.frame.cahier.timbre.entite.Debut;
import org.linotte.frame.cahier.timbre.entite.Pour;
import org.linotte.frame.cahier.timbre.entite.TantQue;
import org.linotte.frame.cahier.timbre.entite.Timbre;
import org.linotte.frame.cahier.timbre.entite.Verbe;
import org.linotte.frame.cahier.timbre.entite.e.Role;
import org.linotte.moteur.xml.analyse.multilangage.Langage;

/**
 * A réécrire, c'est juste un POC écrit très rapidement ...
 * @author CPC
 *
 */
public final class TraducteurTimbreEnTexte {

	private static final String CHAR_RETOUR = "\n";

	private int numeroLigne = 1;

	private boolean debut = true; // Pour le langage Linotte 2 et Linotte 1

	public String traiter(List<Timbre> timbresAExecuter, Langage langage) {
		StringBuilder livre = new StringBuilder();
		Debut debutTimbre = null;
		for (Timbre timbreGraphique : timbresAExecuter) {
			if (timbreGraphique instanceof Debut) {
				debutTimbre = (Debut) timbreGraphique;
			}
		}
		ajouterDebutProgramme(livre, langage);
		if (debutTimbre != null) {
			ajouteCommandes(debutTimbre.timbreSuivant, livre, langage);
		}
		if (debut == true) {
			sectionRole(livre, langage);
		}

		return livre.toString();
	}

	/**
	 * @param suivant
	 * @param livre
	 * @param langage
	 * @return
	 */
	private void ajouteCommandes(Timbre suivant, StringBuilder livre, Langage langage) {
		// Construction du programme :
		while (suivant != null && !(suivant instanceof BlocFin)) {
			suivant = ajouterCommande(suivant, livre, langage);
		}
	}

	private Timbre ajouterCommande(Timbre timbre, StringBuilder livre, Langage langage) {
		if (timbre instanceof Commentaire) {
			ajouterCommentaire((Commentaire) timbre, livre, langage);
			return timbre.timbreSuivant;
		} else if (timbre instanceof Acteur) {
			ajouterCreationActeur((Acteur) timbre, livre, langage);
			return timbre.timbreSuivant;
		} else if (timbre instanceof Condition) {
			if (debut == true) {
				sectionRole(livre, langage);
			}
			ajouterCondition((Condition) timbre, livre, langage);
			return ((BlocDebut) timbre).blocfin.timbreSuivant;
		} else if (timbre instanceof TantQue) {
			if (debut == true) {
				sectionRole(livre, langage);
			}
			ajouterTantQue((TantQue) timbre, livre, langage);
			return ((BlocDebut) timbre).blocfin.timbreSuivant;
		} else if (timbre instanceof Pour) {
			if (debut == true) {
				sectionRole(livre, langage);
			}
			ajouterPour((Pour) timbre, livre, langage);
			return ((BlocDebut) timbre).blocfin.timbreSuivant;
		} else if (timbre instanceof Verbe) {
			if (debut == true) {
				sectionRole(livre, langage);
			}
			switch (timbre.parfum) {
			case AFFICHER:
				ajouterAfficher((Verbe) timbre, livre, langage);
				break;
			case DEMANDER:
				ajouterDemander((Verbe) timbre, livre, langage);
				break;
			case VALOIR:
				ajouterValoir((Verbe) timbre, livre, langage);
				break;
			case EFFACERTABLEAU:
				ajouterEffacerTableau((Verbe) timbre, livre, langage);
				break;
			default:
				break;
			}
			return timbre.timbreSuivant;
		}
		return timbre.timbreSuivant;
	}

	private void ajouterCommentaire(Commentaire timbre, StringBuilder livre, Langage langage) {
		livre.append("// " + timbre.texte);
		ajouterRetourChariot(livre, timbre);
	}

	private void ajouterPour(Pour timbre, StringBuilder livre, Langage langage) {
		switch (langage) {
		case Lyre:
			livre.append("pour " + ajouterActeur(timbre.acteur, langage) + ", " + ajouterActeur(timbre.indexDebut, langage) + " à "
					+ ajouterActeur(timbre.indexFin, langage) + " {");
			ajouterRetourChariot(livre, timbre);
			ajouteCommandes(timbre.timbreSuivant, livre, langage);
			livre.append("}");
			ajouterRetourChariot(livre, null);
			break;
		case Linotte1X:
			livre.append("pour " + ajouterActeur(timbre.acteur, langage) + " de " + ajouterActeur(timbre.indexDebut, langage) + " à "
					+ ajouterActeur(timbre.indexFin, langage) + ", lis");
			ajouterRetourChariot(livre, timbre);
			ajouteCommandes(timbre.timbreSuivant, livre, langage);
			livre.append("ferme");
			ajouterRetourChariot(livre, null);
			break;
		case Linotte2:
			livre.append("pour " + ajouterActeur(timbre.acteur, langage) + " de " + ajouterActeur(timbre.indexDebut, langage) + " à "
					+ ajouterActeur(timbre.indexFin, langage) + " lis");
			ajouterRetourChariot(livre, timbre);
			ajouteCommandes(timbre.timbreSuivant, livre, langage);
			livre.append("ferme");
			ajouterRetourChariot(livre, null);
			break;
		case Linnet:
			livre.append("for " + ajouterActeur(timbre.acteur, langage) + " , " + ajouterActeur(timbre.indexDebut, langage) + " to "
					+ ajouterActeur(timbre.indexFin, langage) + " {");
			ajouterRetourChariot(livre, timbre);
			ajouteCommandes(timbre.timbreSuivant, livre, langage);
			livre.append("}");
			ajouterRetourChariot(livre, null);
			break;
		}
	}

	private void ajouterCondition(Condition timbre, StringBuilder livre, Langage langage) {
		switch (langage) {
		case Lyre:
			livre.append("si " + ajouterActeur(timbre.formule, langage) + "  {");
			ajouterRetourChariot(livre, timbre);
			ajouteCommandes(timbre.timbreSuivant, livre, langage);
			livre.append("}");
			ajouterRetourChariot(livre, null);
			if (!(timbre.timbreSuivantSecondaire instanceof BlocFin)) {
				livre.append("sinon {");
				ajouterRetourChariot(livre, null);
				ajouteCommandes(timbre.timbreSuivantSecondaire, livre, langage);
				livre.append("}");
				ajouterRetourChariot(livre, null);
			}
			break;
		case Linotte1X:
			livre.append("si " + ajouterActeur(timbre.formule, langage) + " alors lis");
			ajouterRetourChariot(livre, timbre);
			ajouteCommandes(timbre.timbreSuivant, livre, langage);
			livre.append("ferme");
			ajouterRetourChariot(livre, null);
			if (!(timbre.timbreSuivantSecondaire instanceof BlocFin)) {
				livre.append("sinon, lis");
				ajouterRetourChariot(livre, null);
				ajouteCommandes(timbre.timbreSuivantSecondaire, livre, langage);
				livre.append("ferme");
				ajouterRetourChariot(livre, null);
			}
			break;
		case Linotte2:
			livre.append("si " + ajouterActeur(timbre.formule, langage) + " lis");
			ajouterRetourChariot(livre, timbre);
			ajouteCommandes(timbre.timbreSuivant, livre, langage);
			livre.append("ferme");
			ajouterRetourChariot(livre, null);
			if (!(timbre.timbreSuivantSecondaire instanceof BlocFin)) {
				livre.append("sinon lis");
				ajouterRetourChariot(livre, null);
				ajouteCommandes(timbre.timbreSuivantSecondaire, livre, langage);
				livre.append("ferme");
				ajouterRetourChariot(livre, null);
			}
			break;
		case Linnet:
			livre.append("if " + ajouterActeur(timbre.formule, langage) + " {");
			ajouterRetourChariot(livre, timbre);
			ajouteCommandes(timbre.timbreSuivant, livre, langage);
			livre.append("}");
			ajouterRetourChariot(livre, null);
			if (!(timbre.timbreSuivantSecondaire instanceof BlocFin)) {
				livre.append("else {");
				ajouterRetourChariot(livre, null);
				ajouteCommandes(timbre.timbreSuivantSecondaire, livre, langage);
				livre.append("}");
				ajouterRetourChariot(livre, null);
			}
			break;
		}
	}

	private void ajouterTantQue(TantQue timbre, StringBuilder livre, Langage langage) {
		switch (langage) {
		case Lyre:
			livre.append("tant que " + ajouterActeur(timbre.formule, langage) + "  {");
			ajouterRetourChariot(livre, timbre);
			ajouteCommandes(timbre.timbreSuivant, livre, langage);
			livre.append("}");
			ajouterRetourChariot(livre, null);
			break;
		case Linotte1X:
			livre.append("tant que " + ajouterActeur(timbre.formule, langage) + " , lis");
			ajouterRetourChariot(livre, timbre);
			ajouteCommandes(timbre.timbreSuivant, livre, langage);
			livre.append("ferme");
			ajouterRetourChariot(livre, null);
			break;
		case Linotte2:
			livre.append("tant que " + ajouterActeur(timbre.formule, langage) + " lis");
			ajouterRetourChariot(livre, timbre);
			ajouteCommandes(timbre.timbreSuivant, livre, langage);
			livre.append("ferme");
			ajouterRetourChariot(livre, null);
			break;
		case Linnet:
			livre.append("while " + ajouterActeur(timbre.formule, langage) + " {");
			ajouterRetourChariot(livre, timbre);
			ajouteCommandes(timbre.timbreSuivant, livre, langage);
			livre.append("}");
			ajouterRetourChariot(livre, null);
			break;
		}
	}

	private void sectionRole(StringBuilder livre, Langage langage) {
		debut = false;
		switch (langage) {
		case Linnet:
		case Lyre:
			break;
		case Linotte1X:
			livre.append("actions :");
			ajouterRetourChariot(livre, null);
			break;
		case Linotte2:
			livre.append("début");
			ajouterRetourChariot(livre, null);
			break;
		default:
			break;
		}
	}

	private void ajouterValoir(Verbe timbre, StringBuilder livre, Langage langage) {
		switch (langage) {
		case Linnet:
		case Lyre:
			livre.append(ajouterActeur(timbre.parametres[0], langage)).append(" = ").append(ajouterActeur(timbre.parametres[1], langage));
			ajouterRetourChariot(livre, timbre);
			break;
		case Linotte1X:
		case Linotte2:
			livre.append(ajouterActeur(timbre.parametres[0], langage)).append(" vaut ").append(ajouterActeur(timbre.parametres[1], langage));
			ajouterRetourChariot(livre, timbre);
			break;
		}
	}

	private void ajouterEffacerTableau(Verbe timbre, StringBuilder livre, Langage langage) {
		switch (langage) {
		case Linnet:
			livre.append("clean");
			ajouterRetourChariot(livre, timbre);
			break;
		case Lyre:
			livre.append("efface");
			ajouterRetourChariot(livre, timbre);
			break;
		case Linotte1X:
			livre.append("efface le tableau");
			ajouterRetourChariot(livre, timbre);
			break;
		case Linotte2:
			livre.append("efface tableau");
			ajouterRetourChariot(livre, timbre);
			break;
		}

	}

	private void ajouterCreationActeur(Acteur timbre, StringBuilder livre, Langage langage) {
		switch (langage) {
		case Linnet:
		case Lyre:
			livre.append(timbre.texte);
			livre.append(" := ");
			livre.append(pasNulle(timbre.valeur, timbre.role));
			ajouterRetourChariot(livre, timbre);
			break;
		case Linotte1X:
		case Linotte2:
			livre.append(timbre.texte);
			livre.append(" est un ");
			livre.append(timbre.role == Role.NOMBRE ? "nombre" : "texte");
			if (timbre.valeur != null) {
				livre.append(" valant ");
				livre.append(pasNulle(timbre.valeur, timbre.role));
			}
			ajouterRetourChariot(livre, timbre);
			break;
		default:
			break;
		}
	}

	private void ajouterDemander(Verbe timbre, StringBuilder livre, Langage langage) {
		switch (langage) {
		case Linnet:
			livre.append("input ").append(ajouterActeur(timbre.parametres[0], langage));
			ajouterRetourChariot(livre, timbre);
			break;
		case Linotte1X:
		case Linotte2:
		case Lyre:
			livre.append("demande ").append(ajouterActeur(timbre.parametres[0], langage));
			ajouterRetourChariot(livre, timbre);
			break;
		}
	}

	private void ajouterAfficher(Verbe timbre, StringBuilder livre, Langage langage) {
		switch (langage) {
		case Linnet:
			livre.append("print ").append(ajouterActeur(timbre.parametres[0], langage));
			ajouterRetourChariot(livre, timbre);
			break;
		case Linotte1X:
		case Linotte2:
		case Lyre:
			livre.append("affiche ").append(ajouterActeur(timbre.parametres[0], langage));
			ajouterRetourChariot(livre, timbre);
			break;
		}
	}

	private String ajouterActeur(Acteur acteur, Langage langage) {
		StringBuilder text = new StringBuilder();
		if (acteur != null) {
			if (acteur.texte == null) { // Anomyme :
				// ROLE
				switch (acteur.role) {
				default:
					text.append(pasNulle(acteur.valeur, acteur.role));
					break;
				}
			} else {
				text.append(acteur.texte); // Nom de l'acteur
			}
			return text.toString();

		} else
			return "";
	}

	private String pasNulle(String valeur, Role role) {
		if (valeur == null) {
			switch (role) {
			case NOMBRE:
				return "0";
			case TEXTE:
				return "\"\"";
			default:
				return "";
			}
		} else {
			switch (role) {
			case TEXTE:
				return "\"" + valeur + "\"";
			default:
				return valeur;
			}
		}
	}

	private void ajouterRetourChariot(StringBuilder livre, Timbre suivant) {
		livre./*append("//").append(numeroLigne).*/append(CHAR_RETOUR);
		if (suivant != null)
			suivant.numeroLigne = numeroLigne;
		numeroLigne++;
	}

	private void ajouterDebutProgramme(StringBuilder livre, Langage langage) {
		if (langage == Langage.Linotte2) {
			livre.append("principale :");
			ajouterRetourChariot(livre, null);
			return;
		} else if (langage == Langage.Linotte1X) {
			livre.append("livre graphique :");
			ajouterRetourChariot(livre, null);
			livre.append("principale :");
			ajouterRetourChariot(livre, null);
			livre.append("rôles :");
			ajouterRetourChariot(livre, null);
			return;
		}
	}

}
